% Calculation based on Cauchy strain
% left end fixed and external force acts downward to the center of right end

% R[V[c݂ɂvZ
% r[̍[ŒDE[̒ɉɗ͂D

% g, cm, sec

addpath('../two_dim_fea');

m = 11; n = 3;
thickness = 1;

[ points, triangles ] = rectangular_object( m, n, 10, 2 );

% E = 0.1 MPa; rho = 0.020 g/cm^2
Young = 1.0*1e+6; nu = 0.48; density = 0.020;
% 10.0 N --> fpush = 10.0*1e+5;
force = [0; -1.2*1e+5];

[ lambda, mu ] = Lame_constants( Young, nu );

npoints = size(points,2);
ntriangles = size(triangles,1);
elastic = Body(npoints, points, ntriangles, triangles, thickness);
elastic = elastic.mechanical_parameters(density, lambda, mu);

disps_natural = zeros(2,npoints);
un_natural = reshape(disps_natural, [2*npoints,1]);

internal_energy = @(un) internal_energy_params( un, npoints, elastic, force );

% fix point 1 1+m 1+2m
% a set of constraints: A' un = b (each column specifies one constraint)
A = zeros(2*npoints, 6); b = zeros(6,1);
k = 1;     A(2*k-1, 1) = 1; A(2*k, 2) = 1;
k = 1+m;   A(2*k-1, 3) = 1; A(2*k, 4) = 1;
k = 1+2*m; A(2*k-1, 5) = 1; A(2*k, 6) = 1;

options = optimset('Display','iter', 'MaxFunEvals',3*10^5);
[un_Cauchy, Emin_Cauchy] = fmincon(internal_energy, un_natural, [], [], A', b, [], [], [], options);

figure('position', [0, 0, 800, 800]);
set(0,'defaultAxesFontSize',32);
set(0,'defaultTextFontSize',32);

clf;
elastic.draw( reshape(un_Cauchy, [2,npoints]) );
xlim([0,12]); ylim([-10,2]); grid on;
pbaspect([1 1 1]);
saveas(gcf, 'beam_deformed_external_force_Cauchy_strain.png');

% calculating nodal force vector
elastic = elastic.calculate_stiffness_matrix;
disps = reshape(un_Cauchy, [2,npoints]);
forces_Cauchy = elastic.nodal_forces_Cauchy_strain(disps);
forces_Cauchy

% calculating nodal force vector (simpler)
%elastic = elastic.calculate_stiffness_matrix;
%forces_Cauchy_2 = -elastic.Stiffness_Matrix*un_Cauchy;
%forces_Cauchy_2

function energy = internal_energy_params( un, npoints, elastic, force )
    disps = reshape(un, [2,npoints]);
    energy = elastic.total_strain_potential_energy(disps);
    energy = energy - force'*disps(:,22);
end
