% g, cm, sec

dt0 = datetime;
addpath('../two_dim_fea');

% p = 2e+4 = 0.02e+6 --> 0.002 MPa = 2 KPa
%pressure = 0.5e+4; filename = 'pneu_net_actuator_deformed_500kPa.png';  % Emin = -8.3180e+05;
%pressure = 0.6e+4; filename = 'pneu_net_actuator_deformed_600kPa.png';  % Emin = -1.0022e+06
pressure = 0.7e+4; filename = 'pneu_net_actuator_deformed_700kPa.png';  % Emin = -1.1800e+06

[points,triangles] = pneu_net_actuator_geometry();
npoints = size(points,2); ntriangles = size(triangles,1);
pneu = Body(npoints, points, ntriangles, triangles, 1);

% E = 0.1 MPa; c = 0.04 kPa s; rho = 1 g/cm^2
Young = 1.0*1e+6; c = 0.4*1e+3; nu = 0.48; density = 1.00;
[ lambda, mu ] = Lame_constants( Young, nu );
pneu = pneu.mechanical_parameters(density, lambda, mu);
pneu = pneu.calculate_stiffness_matrix;

figure('position', [0, 0, 800, 800]);
set(0,'defaultAxesFontSize',32);
set(0,'defaultTextFontSize',32);

clf;
pneu.draw;
xlim([-15,25]); ylim([-3,37]);
yticks(0:10:30);
pbaspect([1 1 1]);
grid on;
drawnow;
saveas(gcf, 'pneu_net_actuator_natural.png');

[ outer, inner ] = pneu.outer_and_inner_contours;
edges = pneu.Contours(inner).Edges; % assume one inner countour
index_pressure_area = [ edges(1,1), edges(:,2)' ];
pneu.surrounded_area_fill(index_pressure_area);
drawnow;

index_floor = [33, 213, 214, 215, 216, 123, 124, 125];
A = pneu.constraint_matrix(index_floor);
b = zeros(2*8,1);
points(:, index_floor)

length_100_153 = 4.00;  % length between P100 & P153
length_105_183 = 4.00;  % length between P105 & P183

disps_natural = zeros(2,npoints);
un_natural = reshape(disps_natural, [2*npoints,1]);

internal_energy = @(sol) internal_energy_params( sol, pneu, pressure, inner );
sol_init = [ un_natural; zeros(2*8,1); 0; 0 ];
nonlcon = @(sol) nonlcon_params( sol, pneu, pressure, inner, A, b, length_100_153, length_105_183 );
options = optimset('Display','iter', 'MaxFunEvals',5*10^5);
[sol_deformed, Emin] = fmincon(internal_energy, sol_init, [], [], [], [], [], [], nonlcon, options);
un_deformed = sol_deformed(1:2*npoints);

dt1 = datetime;
dt01 = between(dt0, dt1);
dt01

Emin
clf;
pneu.draw( reshape(un_deformed, [2,npoints]) );
xlim([-15,25]); ylim([-3,37]);
yticks(0:10:30);
pbaspect([1 1 1]);
grid on;
saveas(gcf, filename);

function energy = internal_energy_params( sol, pneu, pressure, inner )
    persistent npoints index_pressure_area count
    if isempty(npoints)
        npoints = pneu.numNodalPoints;
        edges = pneu.Contours(inner).Edges; % assume one inner countour
        index_pressure_area = [ edges(1,1), edges(:,2)' ];
        count = 0;
    end
    count = count + 1;
    
    un = sol(1:2*npoints);
    disps = reshape(un, [2,npoints]);
    
    %if mod(count,1000) == 0
    %    count
    %    elastic.draw(disps);
    %    drawnow;
    %end
    
    energy = pneu.total_strain_potential_energy(disps);
    area = pneu.surrounded_area(index_pressure_area, disps(:,index_pressure_area));
    % area < 0 for inner contour
    volume = pneu.Thickness * (-area);
    energy = energy - pressure*volume;
end

function [cineq, ceq] = nonlcon_params( sol, pneu, pressure, inner, A, b, length_100_153, length_105_183 )
    persistent npoints K count
    if isempty(npoints)
        npoints = pneu.numNodalPoints;
        K = pneu.Stiffness_Matrix;
        count = 0;
    end
    count = count + 1;
    
    cineq = [];
    
    un = sol(1:2*npoints);
    lambda = sol(2*npoints+1:2*npoints+2*8);
    lambda_d1 = sol(2*npoints+2*8+1);
    lambda_d2 = sol(2*npoints+2*8+2);
    
    disps = reshape(un, [2,npoints]);
    positions = pneu.positional_vectors(disps);
    nodal_forces = pneu.nodal_forces_equivalent_to_pressure(inner, positions, pressure);
    fn = reshape(nodal_forces, [2*npoints,1]);
    
    d_100_153 = distance_two_nodal_points(pneu, 100, 153, disps);
    d_105_183 = distance_two_nodal_points(pneu, 105, 183, disps);
    d_100_153_un = zeros(2*npoints,1);
    d_100_153_un(2*100-1:2*100) = (positions(:,100)-positions(:,153))/d_100_153;
    d_100_153_un(2*153-1:2*153) = (positions(:,153)-positions(:,100))/d_100_153;
    d_105_183_un = zeros(2*npoints,1);
    d_105_183_un(2*105-1:2*105) = (positions(:,105)-positions(:,183))/d_105_183;
    d_105_183_un(2*183-1:2*183) = (positions(:,183)-positions(:,105))/d_105_183;
    
    ceq = [ K*un - A*lambda - d_100_153_un*lambda_d1 - d_105_183_un*lambda_d2 - fn; ...
            -A'*un+b; ...
            -(d_100_153 - length_100_153); ...
            -(d_105_183 - length_105_183) ];
end

function [p,t] = pneu_net_actuator_geometry ()
    [p,t] = half_ring_object(11, 3, 5.00, 3.00, pi/2);
    np = size(p,2); nt = size(t,1);
    
    p1 = p + [ 0.00; 5.00 ]; t1 = t;
    halfring1 = Body(np, p1, nt, t1, 1);
    p2 = p + [ 0.00; 5.00+10.00 ]; t2 = t;
    halfring2 = Body(np, p2, nt, t2, 1);
    p3 = p + [ 0.00; 5.00+2*10.00 ]; t3 = t;
    halfring3 = Body(np, p3, nt, t3, 1);
    
    [rp,rt] = rectangular_object(2, 5, 1.00, 4.00);
    rp1 = rp + [ 0.00; 8.00 ];  rt1 = rt;
    rp2 = rp + [ 0.00; 18.00 ]; rt2 = rt;
    
    [top_p,top_t] = rectangular_object(6, 3, 5.00, 2.00);
    top_p = top_p + [ 0.00; 28.00 ];
    [ right_p, right_t ] = rectangular_object(3, 31, 2.00, 30.00);
    right_p = right_p + [ 5.00; 0.00 ];
    [bottom_p,bottom_t] = rectangular_object(6, 3, 5.00, 2.00);
    
    p = []; t = []; s = [];
    [p,t,s] = combine_objects(p, t, [], p1, t1, []);
    [p,t,s] = combine_objects(p, t, [], p2, t2, []);
    [p,t,s] = combine_objects(p, t, [], p3, t3, []);
    [p,t,s] = combine_objects(p, t, [], rp1, rt1, []);
    [p,t,s] = combine_objects(p, t, [], rp2, rt2, []);
    [p,t,s] = combine_objects(p, t, [], top_p, top_t, []);
    [p,t,s] = combine_objects(p, t, [], right_p, right_t, []);
    [p,t,s] = combine_objects(p, t, [], bottom_p, bottom_t, []);
end

function dist = distance_two_nodal_points (obj, k, l, disps)
    rk = obj.NodalPoints(k).Coordinates + disps(:,k);
    rl = obj.NodalPoints(l).Coordinates + disps(:,l);
    d = rl - rk;
    dist = sqrt(d'*d);
end
